@@ -2,7 +2,7 @@ use std::process::Command;
 
 #[cfg(not(windows))]
 fn get_echo_command() -> Command {
-    Command::new("/bin/echo")
+    Command::new("echo")
 }
 
 #[cfg(windows)]
